from typing import Optional

from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.exception.check_error import CheckError
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class CommandChecks:
    _permissions: Optional[PermissionServiceABC] = None
    _client_utils: Optional[ClientUtilsABC] = None
    _message_service: Optional[MessageServiceABC] = None
    _t: Optional[TranslatePipe] = None

    @classmethod
    def init(
        cls,
        permissions: PermissionServiceABC,
        client_utils: ClientUtilsABC,
        message_service: MessageServiceABC,
        translate: TranslatePipe,
    ):
        cls._permissions = permissions
        cls._client_utils = client_utils
        cls._message_service = message_service
        cls._t = translate

    @classmethod
    def check_is_ready(cls):
        async def check_if_bot_is_ready_yet_and_respond(ctx: Context) -> bool:
            result = await cls._client_utils.check_if_bot_is_ready_yet_and_respond(ctx)
            if not result:
                raise CheckError(f"Bot is not ready")
            return result

        return commands.check(check_if_bot_is_ready_yet_and_respond)

    @classmethod
    def check_is_member_admin(cls):
        async def check_is_member_admin(ctx: Context):
            has_permission = cls._permissions.is_member_admin(ctx.author)
            if not has_permission:
                await cls._message_service.send_ctx_msg(ctx, cls._t.transform("common.no_permission_message"))
                raise CheckError(f"Member {ctx.author.name} is not admin")

            return has_permission

        return commands.check(check_is_member_admin)

    @classmethod
    def check_is_member_technician(cls):
        async def check_is_member_technician(ctx: Context):
            has_permission = cls._permissions.is_member_technician(ctx.author)
            if not has_permission:
                await cls._message_service.send_ctx_msg(ctx, cls._t.transform("common.no_permission_message"))
                raise CheckError(f"Member {ctx.author.name} is not technician")

            return has_permission

        return commands.check(check_is_member_technician)

    @classmethod
    def check_is_member_moderator(cls):
        async def check_is_member_moderator(ctx: Context):
            has_permission = cls._permissions.is_member_moderator(ctx.author)
            if not has_permission:
                await cls._message_service.send_ctx_msg(ctx, cls._t.transform("common.no_permission_message"))
                raise CheckError(f"Member {ctx.author.name} is not moderator")

            return has_permission

        return commands.check(check_is_member_moderator)
