from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.model.client import Client
from bot_data.model.client_history import ClientHistory
from bot_graphql.abc.data_query_with_history_abc import DataQueryWithHistoryABC


class ClientQuery(DataQueryWithHistoryABC):
    def __init__(self, bot: DiscordBotServiceABC, db: DatabaseContextABC):
        DataQueryWithHistoryABC.__init__(self, "Client", "ClientsHistory", ClientHistory, db)

        self._bot = bot

        self.set_field("id", self.resolve_id)
        self.set_field("discordId", self.resolve_discord_id)
        self.set_field("name", self.resolve_name)
        self.set_field("sentMessageCount", self.resolve_sent_message_count)
        self.set_field("receivedMessageCount", self.resolve_received_message_count)
        self.set_field("deletedMessageCount", self.resolve_deleted_message_count)
        self.set_field("receivedCommandCount", self.resolve_received_command_count)
        self.set_field("movedUsersCount", self.resolve_moved_users_count)
        self.set_field("server", self.resolve_server)

    @staticmethod
    def resolve_id(client: Client, *_):
        return client.id

    @staticmethod
    def resolve_discord_id(client: Client, *_):
        return client.discord_id

    @staticmethod
    def resolve_name(client: Client, *_):
        return client.name

    @staticmethod
    def resolve_sent_message_count(client: Client, *_):
        return client.sent_message_count

    @staticmethod
    def resolve_received_message_count(client: Client, *_):
        return client.received_message_count

    @staticmethod
    def resolve_deleted_message_count(client: Client, *_):
        return client.deleted_message_count

    @staticmethod
    def resolve_received_command_count(client: Client, *_):
        return client.received_command_count

    @staticmethod
    def resolve_moved_users_count(client: Client, *_):
        return client.moved_users_count

    @staticmethod
    def resolve_server(client: Client, *_):
        return client.server
