from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_data.model.server_config_history import ServerConfigHistory
from bot_data.model.team_member_type_enum import TeamMemberTypeEnum
from bot_graphql.abc.data_query_with_history_abc import DataQueryWithHistoryABC


class ServerConfigQuery(DataQueryWithHistoryABC):
    def __init__(self, db: DatabaseContextABC):
        DataQueryWithHistoryABC.__init__(self, "ServerConfig", "CFG_ServerHistory", ServerConfigHistory, db)

        self.set_field("id", lambda config, *_: config.id)
        self.set_field("messageDeleteTimer", lambda config, *_: config.message_delete_timer)
        self.set_field("notificationChatId", lambda config, *_: config.notification_chat_id)
        self.set_field("maxVoiceStateHours", lambda config, *_: config.max_voice_state_hours)
        self.set_field("xpPerMessage", lambda config, *_: config.xp_per_message)
        self.set_field("xpPerReaction", lambda config, *_: config.xp_per_reaction)
        self.set_field("maxMessageXpPerHour", lambda config, *_: config.max_message_xp_per_hour)
        self.set_field("xpPerOntimeHour", lambda config, *_: config.xp_per_ontime_hour)
        self.set_field("xpPerEventParticipation", lambda config, *_: config.xp_per_event_participation)
        self.set_field("xpPerAchievement", lambda config, *_: config.xp_per_achievement)
        self.set_field("afkCommandChannelId", lambda config, *_: config.afk_command_channel_id)
        self.set_field("helpVoiceChannelId", lambda config, *_: config.help_voice_channel_id)
        self.set_field("teamChannelId", lambda config, *_: config.team_channel_id)
        self.set_field("loginMessageChannelId", lambda config, *_: config.login_message_channel_id)
        self.add_collection(
            "featureFlag",
            lambda config, *_: List(any, [{"key": x, "value": config.feature_flags[x]} for x in config.feature_flags]),
        )
        self.set_field("afkChannelIds", lambda config, *_: config.afk_channel_ids)
        self.set_field(
            "moderatorRoleIds",
            lambda config, *_: config.team_role_ids.where(
                lambda x: x.team_member_type == TeamMemberTypeEnum.moderator
            ).select(lambda x: x.role_id),
        )
        self.set_field(
            "adminRoleIds",
            lambda config, *_: config.team_role_ids.where(
                lambda x: x.team_member_type == TeamMemberTypeEnum.admin
            ).select(lambda x: x.role_id),
        )
        self.set_field("server", lambda config, *_: config.server)
